import logging
import time
from typing import Callable

from ddns_manager.ddns_updater import DDNSUpdater
from ddns_manager.public_ip_getter import PublicIPGetter


class DDNSManager:
    def __init__(self, ip_getter: PublicIPGetter, ddns_updater: DDNSUpdater):
        self.__ddns_updater = ddns_updater
        self.__ip_getter = ip_getter
        self.__current_ip = None
        self.__update_current_ip()
        self.__ddns_ip = None

    @property
    def current_ip(self):
        self.__update_current_ip()
        return self.__current_ip

    @property
    def ddns_ip(self):
        return self.__ddns_ip

    def is_ddns_outdated(self, update_current_ip: bool = True):
        if update_current_ip:
            self.__update_current_ip()
        return self.__ddns_ip != self.__current_ip

    def update_ddns(self, update_current_ip: bool = True):
        if self.is_ddns_outdated(update_current_ip=update_current_ip):
            self.__update_ddns()

    def update_loop(self, interval: int, on_error: Callable[[Exception], None]):
        while True:
            try:
                logging.info("Starting new update routine.")
                self.update_ddns(update_current_ip=True)
            except Exception as e:
                on_error(e)
            finally:
                time.sleep(interval)

    def __update_current_ip(self):
        self.__current_ip = self.__ip_getter.get_current_ip()

    def __update_ddns(self):
        self.__ddns_updater.update_ddns_record(ip=self.__current_ip)
        self.__ddns_ip = self.__current_ip



"""Setup script for ddns-manager"""

import os.path
from setuptools import setup

# The directory containing this file
HERE = os.path.abspath(os.path.dirname(__file__))

# The text of the README file
with open(os.path.join(HERE, "README.md")) as fid:
    README = fid.read()

# This call to setup() does all the work
setup(
    name="ddns-manager",
    version="0.1.0",
    description="Keep your ddns up to date",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/hbontempo-br/ddns-manager",
    author="hbontempo",
    author_email="me@hbontempo.dev",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
    ],
    packages=["ddns_manager"],
    include_package_data=True,
    install_requires=[
        "click", "PyYAML", "requests", "retry"
    ],
    entry_points={
        "console_scripts": [
            "ddns_manager=ddns_manager.__main__:cli",
        ]
    },
)